@@ -148,5 +148,8 @@ from pytensor.tensor.type import *  # noqa
 from pytensor.tensor.type_other import *  # noqa
 from pytensor.tensor.var import TensorConstant, TensorVariable  # noqa
 
+# Allow accessing numpy constants from pytensor.tensor
+from numpy import e, euler_gamma, inf, infty, nan, newaxis, pi  # noqa
+
 
 __all__ = ["random"]  # noqa: F405
