@@ -146,14 +146,7 @@ def shape(x: Union[np.ndarray, Number, Variable]) -> Variable:
     if not isinstance(x, Variable):
         x = at.as_tensor_variable(x)
 
-    x_type = x.type
-
-    if isinstance(x_type, TensorType) and all(s is not None for s in x_type.shape):
-        res = at.as_tensor_variable(x_type.shape, ndim=1, dtype=np.int64)
-    else:
-        res = _shape(x)
-
-    return res
+    return _shape(x)
 
 
 @_get_vector_length.register(Shape)
@@ -395,6 +388,7 @@ class SpecifyShape(COp):
     view_map = {0: [0]}
     __props__ = ()
     _f16_ok = True
+    _output_type_depends_on_input_value = True
 
     def make_node(self, x, *shape):
         from pytensor.tensor.basic import get_underlying_scalar_constant_value
@@ -594,6 +588,7 @@ class Reshape(COp):
 
     view_map = {0: [0]}  # output 0 is potentially aliased to inputs [0]
     _f16_ok = True
+    _output_type_depends_on_input_value = True
 
     check_input = False
     __props__ = ("ndim",)
